from pymongo import MongoClient
import datetime
import os

gmt_to_ist = datetime.timedelta(hours=5, minutes=30)


def get_database():
    server, port = os.environ['DATABASE_SERVER'].rsplit(':', 1)
    client = MongoClient(server, port=int(port))
    database = client[os.environ['DATABASE_CLIENT']]
    return database


def get_all_users():
    """
    :return: list of all users in database
    """
    database = get_database()
    collection = database['users']
    data = collection.find({})
    return list(x for x in data)


def get_running_trips():
    """
    :return: list of all trips that are running
    """
    database = get_database()
    collection = database['trips']
    data = collection.find({'running': True, 'user': {
        '$nin': os.environ['BLACKLIST_CLIENTS'].split(',')
    }, 'client_client': {
        '$nin': os.environ['BLACKLIST_CLIENT_CLIENT'].split(',')
    }})
    return list(x for x in data)


def get_trips(user, client, start, end):
    """
    :param user: username
    :param client: client name
    :param start: start time ex: [2019, 01, 01]
    :param end:  End Time ex: [2019, 02, 02]
    :return: list of all the trips with given client name else return all the trips with user
    """
    database = get_database()
    collection = database['trips']
    start = datetime.datetime(start[2], start[1], start[0]) - gmt_to_ist
    end = datetime.datetime(end[2], end[1], end[0]) - gmt_to_ist

    query = {
        'user': user,
        '$and': [{
            '$or': [{
                'startTime': {'$lte': end}
            }, {
                'startTime': {'$lte': end.isoformat()}
            }]
        }, {
            '$or': [{
                'endTime': {'$gte': start}
            }, {
                'endTime': {'$gte': start.isoformat()}
            }, {
                'running': True
            }]
        }
        ]
    }
    if client == '' or client is None:
        data = collection.find(query)
    else:
        query['client_client'] = client
        data = collection.find(query)
    if isinstance(data, list):
        return data
    else:
        res = list()
        for x in data:
            res.append(x)
        return res


def get_all_pings(trips_list):
    """
    get all pings for all the trips
    :param trips_list: list of all the trips Id's
    :return: Object containing list of ['_id':'tripId', 'pings': list( all pings )]
    """
    trips_ids = [x['_id'] for x in trips_list]
    database = get_database()
    collection = database['status']
    try:
        data = collection.aggregate([{
            '$match': {'tripId': {'$in': trips_ids}}
        }, {
            '$group': {'_id': '$tripId', 'pings': {'$push': '$$ROOT'}}
        }], allowDisUse=True)
        return list(x for x in data)
    except Exception as e:
        print(str(e))
        return []


def get_pings(trips_list, start, end):
    """
    get all pings for all the trips that have created at between start and end
    :param trips_list: list of all the trips Id's
    :param start: start time for the trips
    :param end: end time for the trips
    :return: Object containing list of ['_id':'tripId', 'pings': list( all pings )]
    """
    start = datetime.datetime(start[2], start[1], start[0]) - gmt_to_ist
    end = datetime.datetime(end[2], end[1], end[0]) - gmt_to_ist + datetime.timedelta(1)
    database = get_database()
    collection = database['status']
    data = collection.aggregate([{
        '$match': {'tripId': {'$in': trips_list}, 'createdAt': {'$gte': start, '$lte': end}}
    }, {
        '$group': {'_id': '$tripId', 'pings': {'$push': '$$ROOT'}}
    }], allowDiskUse=True)
    return list(x for x in data)



from datetime import datetime

time_fmt = "%Y-%m-%dT%H:%M:%S.%fz"


def get_createdAt(trip):
    """
    :param trip: trip Object
    :return: createdAt datetime object
    """
    time = trip['createdAt']
    if isinstance(time, str):
        time = datetime.strptime(time, time_fmt)
    return time


def get_startTime(trip):
    """
    :param trip: trip Object
    :return: startTime if startTime exists else createdAt datetime object
    """
    if 'startTime' in trip.keys():
        time = trip['startTime']
        if isinstance(time, str):
            time = datetime.strptime(time, time_fmt)
        return time
    else:
        return get_createdAt(trip)


def get_eta_days(trip):
    if 'eta_days' in trip.keys():
        eta_days = trip['eta_days']
        eta_days = str(eta_days) or eta_days  # to remove that None shit in the eta_days
        try:
            return float(eta_days)
        except Exception as e:
            # print("ERR {0} {1} in {2}".format(e, 'eta_days', trip['_id']))
            return None
    else:
        # print("ERR No {0} in {1}".format('eta_days', trip['_id']))
        return None


def get_eta_hrs(trip):
    eta_days = get_eta_days(trip)
    if eta_days is None:
        if 'eta_hrs' in trip.keys():
            eta_hrs = str(trip['eta_hrs']) or trip['eta_hrs']  # to remove that None shit in the eta_hrs
            try:
                return float(eta_hrs)
            except Exception as e:
                print("ERR {} For Trip {}".format(e, trip['_id']))
    if eta_days is not None:
        return eta_days * 24
    else:
        try:
            return trip['base_google_time'] / 3600
        except Exception as e:
            print(e)
            return -1


def get_ping_rate(trip):
    """
    :param trip: Trip Object
    :return: ping rate in milliseconds if 'pingrate' exists in trip else 3600000 as default
    """
    if 'pingrate' in trip.keys():
        return int(trip['pingrate'])
    else:
        return 3600000


def get_source(trip):
    """
    Retutn the source name for the trip
    :param trip: trip Object
    :return: string containing source name
    """
    if 'srcname' in trip.keys():
        return trip['srcname']
    if 'src' in trip.keys():
        if isinstance(trip['src'], list):
            return trip['src'][0]
        elif isinstance(trip['src'], str):
            return trip['src']
        else:
            return ''
    else:
        return ''


def get_operator(trip):
    """
    Return the operator name for the trip
    :param trip: Object
    :return: operator name
    """
    operator = ''
    try:
        if 'consent' in trip.keys():
            if 'result' in trip['consent']:
                if 'operator' in trip['consent']['result']:
                    operator = trip['consent']['result']['operator']
    except Exception as e:
        operator = ''
        print(e)
    return operator


def get_truck_number(trip):
    """
    Return the Truck number
    :param trip: Object
    :return: string containing truck number
    """
    truck_number = ''
    if 'truck_number' in trip.keys():
        truck_number = trip['truck_number']
    if truck_number == '':
        truck_number = trip['vehicle']
    return truck_number


def get_running(trip):
    """
    Return True if trip is running else False
    :param trip: Object
    :return: trip['running']
    """
    return trip['running']


def get_endTime(trip):
    """
    Return the time when trip is ended
    :param trip: Object
    :return: datetime object for trip end Time
    """
    if not get_running(trip):
        trip_keys = trip.keys()
        if 'endTime' in trip_keys:
            end = trip['endTime']
        elif 'end_time' in trip_keys:
            end = trip['end_time']
        elif 'forcedEndTime' in trip_keys:
            end = trip['forcedEndTime']
        else:
            raise Exception('ERR = Parameter for End Time is not defined')
    else:
        end = datetime.now()
    return end



import math


def square(x):
    """
    Return square of a number
    :param x:
    :return: x * x
    """
    return x * x


def get_distance(src, dest):
    """
    Returns the distance between source and destination in meters
    :param src: [lat, lng]
    :param dest: [lat, lng]
    :return: Distance in meters between src and dest
    """
    radius = 6371000  # Radius of the earth in m
    d_lat = math.radians(dest[0] - src[0])
    d_lon = math.radians(dest[1] - src[1])
    a = square(math.sin(d_lat / 2)) + math.cos(math.radians(src[0])) * math.cos(math.radians(dest[0])) * square(
        math.sin(d_lon / 2))
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
    return math.floor(radius * c)  # Distance in m


def get_days(diff):
    """
    Returns floating point number for days
    :param diff: timedelta
    :return: floating number
    """
    return diff.total_seconds() / (24 * 60 * 60)


def get_days(start, end):
    """
    :param start: start datetime object
    :param end: end datetime object
    :return: time difference in days as float
    """
    return get_days(end - start)



from setuptools import setup, find_packages

setup(
    name='IntugineHelper',
    version='0.8',
    author="Rishabhdeep Singh",
    author_email="rishabhdeepsingh98@gmail.com",
    description="A Helper for Intugine",
    long_description=open('README.md', 'r').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/rishabhdeepsingh/IntugineHelper",
    packages=find_packages(),
    install_requires=[
        'pymongo',
        'dnspython',
        'datetime'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
